module MCR (
	input logic reset,
	input logic clk,
	input logic load_mcr,
	input logic [31:0] mcr,
	input logic [2:0] receive_in_rcblk,
	input logic [2:0] transmit_in_xcblk,	
	input logic load_rcblk,
	input logic load_xcblk,
	input logic read_d_mcr_r,
	input logic read_d_mcr_t,
	output logic [31:0] dynamic_reg_rmult_mcr,
	output logic [31:0] dynamic_reg_tmult_mcr,
	output logic [31:0] rb_out_mcr
);

logic [31:0] mcr_value;

always_ff @(posedge clk) begin
	if(!reset) begin
		mcr_value = 0;
		rb_out_mcr <= 0;
		dynamic_reg_rmult_mcr <= 0;
		dynamic_reg_tmult_mcr <= 0;
	end
	else if (load_mcr) begin
		rb_out_mcr <= mcr;		
		mcr_value = mcr;		
		if(read_d_mcr_t) dynamic_reg_tmult_mcr <= mcr_value;
		else dynamic_reg_tmult_mcr <= 0;
		if(read_d_mcr_r) dynamic_reg_rmult_mcr <= mcr_value;
		else dynamic_reg_rmult_mcr <= 0;
	end
	else if(load_rcblk || load_xcblk) begin	
		if(load_rcblk) begin
			mcr_value[4:2] = receive_in_rcblk; //rcblk writting
		end
		if(load_xcblk) begin
			mcr_value[20:18] = transmit_in_xcblk; //xcblk writting
		end
		rb_out_mcr <= mcr_value;
		if(read_d_mcr_t) dynamic_reg_tmult_mcr <= mcr_value;
		else dynamic_reg_tmult_mcr <= 0;
		if(read_d_mcr_r) dynamic_reg_rmult_mcr <= mcr_value;
		else dynamic_reg_rmult_mcr <= 0;
	end
	else begin
		rb_out_mcr <= mcr_value;
		if(read_d_mcr_t) dynamic_reg_tmult_mcr <= mcr_value;
		else dynamic_reg_tmult_mcr <= 0;
		if(read_d_mcr_r) dynamic_reg_rmult_mcr <= mcr_value;
		else dynamic_reg_rmult_mcr <= 0;
	end
end

endmodule