module control_rb (
        input logic clk, reset,
        input logic [31:0] rb_in_address,
        input logic [31:0] rb_in_write_data,
        input logic rb_in_read,
        input logic rb_in_write,
        input logic rb_in_begin_transfer,
        input logic [3:0] rb_in_byte_enable, 
	
        output wire rb_in_wait_request, 
	  	  
        output logic ac_srg_active,
        output logic ac_companding_active,
        output logic ac_receive_mult_active,
        output logic ac_general_rec_active,
        output logic ac_general_trans_active,
        output logic ac_transmit_active,
        output logic ac_receive_active,
        output logic ac_transmit_mult_active,
	
        output logic load_spcr,
        output logic load_rcr,
        output logic load_xcr,
        output logic load_mcr,
        output logic load_pcr,
        output logic load_srgr,
        output logic load_xcere0,
        output logic load_xcere1,
        output logic load_xcere2,
        output logic load_xcere3,
        output logic load_rcere0,
        output logic load_rcere1,
        output logic load_rcere2,
        output logic load_rcere3,
	
        output logic load_rrdy,
        output logic load_rfull,
        output logic load_rsyncerr,
        output logic load_rcblk,
	  
        output logic load_xsyncerr,
        output logic load_xempty,
        output logic load_xrdy,
        output logic load_xcblk,
	
        output logic read_spcr,
        output logic read_rcr,
        output logic read_xcr,
        output logic read_mcr,
        output logic read_pcr,
        output logic read_srgr,
        output logic read_xcere0,
        output logic read_xcere1,
        output logic read_xcere2,
        output logic read_xcere3,
        output logic read_rcere0,
        output logic read_rcere1,
        output logic read_rcere2,
        output logic read_rcere3,
	
        output logic load_general_pcr,
        
        output logic rmult_request,
        output logic tmult_request,
        output logic read_d_rcere0,
        output logic read_d_xcere0,
        output logic read_d_rcere1,
        output logic read_d_xcere1,
        output logic read_d_rcere2,
        output logic read_d_xcere2,
        output logic read_d_rcere3,
        output logic read_d_xcere3,
        output logic read_d_mcr_t,
        output logic read_d_mcr_r,
        output logic read_d_pcr
        
	
    );

    logic [31:0] spcr;
    logic [31:0] pcr;
    logic [31:0] mcr;
    logic [31:0] srgr;
	
    logic xrst, rrst, grst, frst;
    logic xioen, rioen;
    logic [1:0] xmcm;
    logic xmcme, rmcme, rmcm;
    logic dlb, mode_receive;
	
	
    reg [2:0] state_machine_1, state_machine_2, state_machine_3, state_machine_4, state_machine_5;
    reg [2:0] next_1, next_2, next_3, next_4, next_5;

    parameter [3:0] INIT_STATE = 0,
    DYNAMIC_STATE = 1,
    TRANSMIT_STATE = 1,
    RECEIVE_STATE = 1,
    GENERAL_PURPOSE_STATE = 1,
    GENERAL_PURPOSE_REC_STATE = 2,
    GENERAL_PURPOSE_TRANS_STATE = 3,
    COMPANDING_DLB_STATE = 3,
    COMPANDING_NON_DLB_STATE = 4,
    TRANSMIT_MULT_STATE = 2,
    RECEIVE_MULT_STATE = 2,
    SRG_CLK_START_STATE = 1,
    SRG_FRAME_START_STATE = 2,
	BUSY_STATE = 2,
    SELECT_STATE = 5;
	          	    
    //================ STATE MACHINE 1 ====================//
    /* RODA INICIALIZACAO OU DINAMICO */ 
	
    //================ STATE MACHINE 2 ====================//
    /* DA START NO SRG CLK GENERATION E SRG FRAME GENERATION*/
	
    //================ STATE MACHINE 3 ====================//
    /* RODA RECEIVE OU RECEIVE MULTICHANNEL*/ 
	
    //================ STATE MACHINE 4 ====================//
    /* RODA GENERAL PURPOSE TRANSMIT/GENERAL PURPOSE RECEIVE*/
	
    //================ STATE MACHINE 5 ====================//
    /* RODA TRANSMIT OU TRANSMIT MULTICHANNEL
					OU COMPANDING DLB OU NON-DLB*/ 
	
	
    //avalon       	    
    
	
    //state_machine_selection_1
    always begin
		case(state_machine_1)
			INIT_STATE: begin	                
				xrst = (spcr >>> 16) & 1;
				rrst = spcr & 1;
				grst = (spcr >>> 22) & 1; 
				frst = (spcr >>> 23) & 1;
				dlb = (spcr >>> 15) & 1;
				
				xmcm = (mcr >>> 16) & 3;
				rmcm = mcr & 1;
				xmcme = (mcr >>> 25) & 1;
				rmcme = (mcr >>> 9) & 1;
				
				xioen = (pcr >>> 13) & 1;
				rioen = (pcr >>> 12) & 1;
							  
				
				if(rb_in_begin_transfer) begin
					rb_in_wait_request = 1;
					next_1 <= BUSY_STATE;
				end
				else begin
					rb_in_wait_request = 0;   
					if(xrst || rrst || (!xrst && !rrst && xioen && rioen)
					  || (!xrst && !rrst && dlb)) begin
						next_1 <= DYNAMIC_STATE;
					end
					else next_1 <= INIT_STATE;	  
				end                                 
				
			end
			DYNAMIC_STATE: begin
				xrst = (spcr >>> 16) & 1;
				rrst = spcr & 1;
				grst = (spcr >>> 22) & 1; 
				frst = (spcr >>> 23) & 1;
				dlb = (spcr >>> 15) & 1;
				
				xmcm = (mcr >>> 16) & 3;
				rmcm = mcr & 1;
				xmcme = (mcr >>> 25) & 1;
				rmcme = (mcr >>> 9) & 1;
				
				xioen = (pcr >>> 13) & 1;
				rioen = (pcr >>> 12) & 1;
				
				if(rb_in_begin_transfer) begin
					rb_in_wait_request = 1;
					next_1 <= BUSY_STATE;
				end
				else begin
					rb_in_wait_request = 0;  
					
					if(xrst || rrst || (!xrst && !rrst && xioen && rioen)
					  || (!xrst && !rrst && dlb)) begin
						next_1 <= DYNAMIC_STATE;
					end
					else next_1 <= INIT_STATE;
				
				end    
			end
			BUSY_STATE: begin
				rb_in_wait_request = 0;
				next_1 <= DYNAMIC_STATE;
			end
			default: begin
				rb_in_wait_request = 0;
				next_1 <= INIT_STATE;
			end
		endcase
    end
	
    //state_machine_selection_2
    always_comb begin
        
		case(state_machine_2) 
			SELECT_STATE: begin
				if(grst) next_2 <= SRG_CLK_START_STATE;
				else if(frst) next_2 <= SRG_FRAME_START_STATE;
				else if (!xrst && !rrst && dlb) next_2 <= COMPANDING_DLB_STATE;
				else next_2 <= SELECT_STATE;
			end                 
			default: begin
				next_2 <= state_machine_2;
			end
		endcase
        
    end
	
    //state_machine_selection_3
    always_comb begin
        
		case(state_machine_3)
			SELECT_STATE: begin
				if(rrst && (rmcm || rmcme)) next_3 <= RECEIVE_MULT_STATE;
				else if(rrst) next_3 <= RECEIVE_STATE;
				else if (!rrst && !xrst && dlb) next_3 <= COMPANDING_DLB_STATE;
				else next_3 <= SELECT_STATE;
			end 
			default: begin 
				next_3 <= state_machine_3;
			end
		endcase
        
    end
	
    //state_machine_selection_4
    always_comb begin
        
		case(state_machine_4)
			SELECT_STATE: begin
				if(!xrst && !rrst && xioen && rioen) next_4 <= GENERAL_PURPOSE_STATE;
				else if(!xrst && xioen) next_4 <= GENERAL_PURPOSE_TRANS_STATE;
				else if(!rrst && rioen) next_4 <= GENERAL_PURPOSE_REC_STATE;
				else next_4 <= SELECT_STATE;
			end
			default: begin
				next_4 <= state_machine_4;
			end
		endcase
        
    end
	
    //state_machine_selection_5
    always_comb begin
        
		case(state_machine_5)
			SELECT_STATE:
			if (xrst && ((xmcm != 0) || xmcme)) next_5 <= TRANSMIT_MULT_STATE;
			else if (xrst) next_5 <= TRANSMIT_STATE;
			else if (!xrst && !rrst && dlb) next_5 <= COMPANDING_DLB_STATE;
			else next_5 <= SELECT_STATE;
			default: begin
				next_5 <= state_machine_5;
			end
		endcase
        
    end
	
    //state_machine_5
    always_ff @(posedge clk) begin
        
        if(!reset) begin
            ac_transmit_active <= 0;
            ac_transmit_mult_active <= 0;
            load_xsyncerr <= 0;
            load_xempty <= 0;
            load_xrdy <= 0;
            load_xcblk <= 0;
            read_d_mcr_t <= 0;
            read_d_xcere0 <= 0;
            read_d_xcere1 <= 0;
            read_d_xcere2 <= 0;
            read_d_xcere3 <= 0;
			state_machine_5 <= SELECT_STATE;
        end
        else begin
			state_machine_5 <= next_5;
            case(state_machine_5)
                TRANSMIT_STATE: begin
                    if(xrst) begin
                    	ac_transmit_active <= 1;
                    	load_xsyncerr <= 1;
	                    load_xempty <= 1;
	                    load_xrdy <= 1;    
                    end
                    else begin
                    	ac_transmit_active <= 0;
                    	load_xsyncerr <= 0;
	                    load_xempty <= 0;
	                    load_xrdy <= 0;    
                    end                                    
                end
                TRANSMIT_MULT_STATE: begin
                    if(xrst) begin
                    	ac_transmit_active <= 1;
                    	ac_transmit_mult_active <= 1;
                    	load_xsyncerr <= 1;
	                    load_xempty <= 1;
	                    load_xrdy <= 1;
	                    load_xcblk <= 1;
	                    read_d_mcr_t <= 1;
	                    read_d_xcere0 <= 1;
			            read_d_xcere1 <= 1;
			            read_d_xcere2 <= 1;
			            read_d_xcere3 <= 1;
                    end
                    else begin
                    	ac_transmit_active <= 0;
                    	ac_transmit_mult_active <= 0;
                    	load_xsyncerr <= 0;
	                    load_xempty <= 0;
	                    load_xrdy <= 0;
	                    load_xcblk <= 0;
	                    read_d_mcr_t <= 0;
	                    read_d_xcere0 <= 0;
			            read_d_xcere1 <= 0;
			            read_d_xcere2 <= 0;
			            read_d_xcere3 <= 0;
                    end                    
                end
                COMPANDING_DLB_STATE: begin
                    ac_transmit_active <= 1;
                end
                
                default: begin
                end
            endcase
        end
    end
			
    //state_machine_4
    always_ff @(posedge clk) begin
        
        if (!reset) begin
            ac_general_rec_active <= 0;
            ac_general_trans_active <= 0;
            load_general_pcr <= 0;
            read_d_pcr <= 0;
			state_machine_4 <= SELECT_STATE;
        end
        else begin
			state_machine_4 <= next_4;
            case(state_machine_4)
                GENERAL_PURPOSE_STATE: begin
                    ac_general_rec_active <= 1;
                    ac_general_trans_active <= 1;                    
        			read_d_pcr <= 1;
                    load_general_pcr <= 1;
                end
                GENERAL_PURPOSE_REC_STATE: begin
                    ac_general_rec_active <= 1;
                    load_general_pcr <= 1;
                    read_d_pcr <= 1;
                end
                GENERAL_PURPOSE_TRANS_STATE: begin
                    ac_general_trans_active <= 1;
                    load_general_pcr <= 1;
                    read_d_pcr <= 1;
                end
                default: begin
                end
            endcase
        end
    end
	
    //state_machine_3
    always_ff @(posedge clk) begin
        
        if (!reset) begin
            ac_receive_active <= 0;
            ac_receive_mult_active <= 0;
            load_rrdy <= 0;
            load_rfull <= 0;
            load_rsyncerr <= 0;
            load_rcblk <= 0;      
            read_d_rcere0 <= 0;
            read_d_rcere1 <= 0;
            read_d_rcere2 <= 0;
            read_d_rcere3 <= 0;
            read_d_mcr_r <= 0;
			state_machine_3 <= SELECT_STATE;
        end
        else begin
			state_machine_3 <= next_3;
            case(state_machine_3)
                RECEIVE_MULT_STATE: begin
                	if(rrst) begin
                		ac_receive_active <= 1;
                		ac_receive_mult_active <= 1;
                		load_rrdy <= 1;
	                    load_rfull <= 1;
	                    load_rsyncerr <= 1;
	                    load_rcblk <= 1;
	                    read_d_rcere0 <= 1;
			            read_d_rcere1 <= 1;
			            read_d_rcere2 <= 1;
			            read_d_rcere3 <= 1;
	                    read_d_mcr_r <= 1;
                	end
                    else begin
                    	ac_receive_active <= 0;
                    	ac_receive_mult_active <= 0;
                    	load_rrdy <= 0;
	                    load_rfull <= 0;
	                    load_rsyncerr <= 0;
	                    load_rcblk <= 0;
	                    read_d_rcere0 <= 0;
			            read_d_rcere1 <= 0;
			            read_d_rcere2 <= 0;
			            read_d_rcere3 <= 0;
	                    read_d_mcr_r <= 0;
                    end  
                end                       
                RECEIVE_STATE: begin
                    if(rrst) ac_receive_active <= 1;
                    else ac_receive_active <= 0;
                    load_rrdy <= 1;
                    load_rfull <= 1;
                    load_rsyncerr <= 1;
                end
                COMPANDING_DLB_STATE: begin
                    ac_receive_active <= 1;
                end
                default: begin
                end
            endcase
        end
    end
	
    //state_machine_2
    always_ff @(posedge clk) begin
        
        if(!reset) begin
            ac_srg_active <= 0;
			state_machine_2 <= SELECT_STATE;
        end
        else begin
			state_machine_2 <= next_2;
            case(state_machine_2)
                SRG_CLK_START_STATE: begin
                    if(grst) ac_srg_active <= 1;
                    else ac_srg_active <= 0;
                end       
				SRG_FRAME_START_STATE: begin
					if(frst) ac_srg_active <= 1;
                    else ac_srg_active <= 0;
				end
                COMPANDING_DLB_STATE: begin
                	ac_srg_active <= 1;
                end         
                default: begin
                end
            endcase
        end
    end
	
		
    //state_machine_1
    always_ff @(posedge clk) begin
        
		
        if(!reset) begin
            load_spcr <= 0;
            load_rcr <= 0;
            load_xcr <= 0;
            load_mcr <= 0;
            load_pcr <= 0;
            load_srgr <= 0;
            load_xcere0 <= 0;
            load_xcere1 <= 0;
            load_xcere2 <= 0;
            load_xcere3 <= 0;
            load_rcere0 <= 0;
            load_rcere1 <= 0;
            load_rcere2 <= 0;
            load_rcere3 <= 0;
            read_spcr <= 0;
            read_rcr <= 0;
            read_xcr <= 0;
            read_mcr <= 0;
            read_pcr <= 0;
            read_srgr <= 0;
            read_xcere0 <= 0;
            read_xcere1 <= 0;
            read_xcere2 <= 0;
            read_xcere3 <= 0;
            read_rcere0 <= 0;
            read_rcere1 <= 0;
            read_rcere2 <= 0;
            read_rcere3 <= 0;
			ac_companding_active <= 0;
            rmult_request <= 0;
            tmult_request <= 0;
			state_machine_1 <= INIT_STATE;
        end
        else begin 
			state_machine_1 <= next_1;
            case(state_machine_1)
                INIT_STATE: begin
                	if(!xrst && !rrst && !dlb && !xioen && !rioen) begin
                    	ac_companding_active <= 1;
                    end
                    else ac_companding_active <= 0;
                    if(rb_in_begin_transfer && rb_in_write) begin
                        case(rb_in_address[7:0]) 
                            4'h0: begin //SPCR
                                load_spcr <= 1;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                                spcr = rb_in_write_data;
                            end
                            4'h4: begin //RCR
                                load_rcr <= 1;
                                load_spcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h8: begin //XCR
                                load_xcr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end 
                            8'hC: begin //SRGR
                                load_srgr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h10: begin //MCR
                                load_mcr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 1;
                                tmult_request <= 1;
                                mcr = rb_in_write_data;
                            end 
                            8'h14: begin //RCERE0
                                load_rcere0 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h18: begin //RCERE1
                                load_rcere1 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h1C: begin //RCERE2
                                load_rcere2 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h20: begin //RCERE3
                                load_rcere3 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h24: begin //XCERE0
                                load_xcere0 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h28: begin //XCERE1
                                load_xcere1 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h2C: begin //XCERE2
                                load_xcere2 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h30: begin //XCERE3
                                load_xcere3 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h34: begin //PCR
                                load_pcr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                                pcr = rb_in_write_data;
                            end
                        endcase
                    end
                    else begin
                        load_spcr <= 0;
                        load_rcr <= 0;
                        load_xcr <= 0;
                        load_mcr <= 0;
                        load_pcr <= 0;
                        load_srgr <= 0;
                        load_xcere0 <= 0;
                        load_xcere1 <= 0;
                        load_xcere2 <= 0;
                        load_xcere3 <= 0;
                        load_rcere0 <= 0;
                        load_rcere1 <= 0;
                        load_rcere2 <= 0;
                        load_rcere3 <= 0;
                        rmult_request <= 0;
                        tmult_request <= 0;
                    end
                end
			
                DYNAMIC_STATE: begin 
                	if(!xrst && !rrst && !dlb && !xioen && !rioen) begin
                    	ac_companding_active <= 1;
                    end
                    else ac_companding_active <= 0;
                    if(rb_in_begin_transfer && rb_in_read) begin
                        case(rb_in_address[7:0]) 
                            4'h0: begin //SPCR
                                read_spcr <= 1;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            4'h4: begin //RCR
                                read_rcr <= 1;
                                read_spcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h8: begin //XCR
                                read_xcr <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end 
                            8'hC: begin //SRGR
                                read_srgr <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h10: begin //MCR
                                read_mcr <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end 
                            8'h14: begin //RCERE0
                                read_rcere0 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h18: begin //RCERE1
                                read_rcere1 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h1C: begin //RCERE2
                                read_rcere2 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h20: begin //RCERE3
                                read_rcere3 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                            end
                            8'h24: begin //XCERE0
                                read_xcere0 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h28: begin //XCERE1
                                read_xcere1 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h2C: begin //XCERE2
                                read_xcere2 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h30: begin //XCERE3
                                read_xcere3 <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_pcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end
                            8'h34: begin //PCR
                                read_pcr <= 1;
                                read_spcr <= 0;
                                read_rcr <= 0;
                                read_xcr <= 0;
                                read_mcr <= 0;
                                read_srgr <= 0;
                                read_xcere0 <= 0;
                                read_xcere1 <= 0;
                                read_xcere2 <= 0;
                                read_xcere3 <= 0;
                                read_rcere0 <= 0;
                                read_rcere1 <= 0;
                                read_rcere2 <= 0;
                                read_rcere3 <= 0;
                            end 
                        endcase
                    end
                    else if(rb_in_begin_transfer && rb_in_write) begin
                        case(rb_in_address[7:0]) 
                            4'h0: begin //SPCR
                                load_spcr <= 1;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                                spcr = rb_in_write_data;
                            end
                            4'h4: begin //RCR
                                load_rcr <= 1;
                                load_spcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h8: begin //XCR
                                load_xcr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end 
                            8'hC: begin //SRGR
                                load_srgr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h10: begin //MCR
                                load_mcr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 1;
                                tmult_request <= 1;
                                mcr = rb_in_write_data;
                            end 
                            8'h14: begin //RCERE0
                                load_rcere0 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h18: begin //RCERE1
                                load_rcere1 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h1C: begin //RCERE2
                                load_rcere2 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h20: begin //RCERE3
                                load_rcere3 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h24: begin //XCERE0
                                load_xcere0 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h28: begin //XCERE1
                                load_xcere1 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h2C: begin //XCERE2
                                load_xcere2 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h30: begin //XCERE3
                                load_xcere3 <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_pcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                            end
                            8'h34: begin //PCR
                                load_pcr <= 1;
                                load_spcr <= 0;
                                load_rcr <= 0;
                                load_xcr <= 0;
                                load_mcr <= 0;
                                load_srgr <= 0;
                                load_xcere0 <= 0;
                                load_xcere1 <= 0;
                                load_xcere2 <= 0;
                                load_xcere3 <= 0;
                                load_rcere0 <= 0;
                                load_rcere1 <= 0;
                                load_rcere2 <= 0;
                                load_rcere3 <= 0;
                                rmult_request <= 0;
                                tmult_request <= 0;
                                pcr = rb_in_write_data;
                            end
                        endcase
                    end
                    else begin
                        read_pcr <= 0;
                        read_spcr <= 0;
                        read_rcr <= 0;
                        read_xcr <= 0;
                        read_mcr <= 0;
                        read_srgr <= 0;
                        read_xcere0 <= 0;
                        read_xcere1 <= 0;
                        read_xcere2 <= 0;
                        read_xcere3 <= 0;
                        read_rcere0 <= 0;
                        read_rcere1 <= 0;
                        read_rcere2 <= 0;
                        read_rcere3 <= 0;
                        load_pcr <= 0;
                        load_spcr <= 0;
                        load_rcr <= 0;
                        load_xcr <= 0;
                        load_mcr <= 0;
                        load_srgr <= 0;
                        load_xcere0 <= 0;
                        load_xcere1 <= 0;
                        load_xcere2 <= 0;
                        load_xcere3 <= 0;
                        load_rcere0 <= 0;
                        load_rcere1 <= 0;
                        load_rcere2 <= 0;
                        load_rcere3 <= 0;
                        rmult_request <= 0;
                        tmult_request <= 0;
                    end
                end
				default: begin
				end
            endcase 
        end
    end
	
		

endmodule