/*
module dxr(

  input clkx,
  input reset,
  input copy_out_copy,
  
  
  input  [31:0] dxr_in_write_data,
  input dxr_in_write,
  input dxr_in_begin_transfer,
  input [3:0] dxr_in_byte_enable, 
  
  output logic dxr_in_wait_request,
  output logic rb_out_xrdy,
  output logic [31:0] dxr_out
         
);

	always_ff @(posedge clkx) begin 

		
        if(!reset)
        begin
            dxr_out <= 0;
            rb_out_xrdy <= 1;
            dxr_in_wait_request <= 0;
        end
        else
		begin
			dxr_in_wait_request <= 0;
			if(copy_out_copy)
			begin
				rb_out_xrdy <= 1;
			end

			if(dxr_in_begin_transfer)
			begin
				if(dxr_in_write)
				begin
					dxr_out <= dxr_in_write_data;
					dxr_in_wait_request <= 1;
					rb_out_xrdy <= 0;
				end
			end

		end
	
    end

endmodule
*/

module dxr(

  input clkx,
  input reset,
  input copy_out_copy,
  
  
  input  [31:0] dxr_in_write_data,
  input dxr_in_write,
  input dxr_in_begin_transfer,
  input [3:0] dxr_in_byte_enable, 
  
  output logic dxr_in_wait_request,
  output logic rb_out_xrdy,
  output logic [31:0] dxr_out
         
);

	parameter BEGIN_TRANSFER_STATE = 0,
			  BUSY_STATE = 1;
			
	reg avalon_state,next;

	always_comb begin
		case(avalon_state)
		BEGIN_TRANSFER_STATE: begin
			if(dxr_in_begin_transfer) begin
				dxr_in_wait_request <= 1;
				next <= BUSY_STATE;
			end
			else begin
				dxr_in_wait_request <= 0;
				next <= BEGIN_TRANSFER_STATE;
				
			end
		end
		BUSY_STATE: begin
			dxr_in_wait_request <= 0;
			next <= BEGIN_TRANSFER_STATE;
		end
		endcase
	end
     	 
    always_ff @(posedge clkx) begin 		
        if(!reset)
        begin
            dxr_out <= 0;
            rb_out_xrdy <= 1;
        end
        else
		begin
			avalon_state <= next;
			case(avalon_state)
			BEGIN_TRANSFER_STATE: begin
				if(copy_out_copy)
				begin
					rb_out_xrdy <= 1;
				end

				if(dxr_in_begin_transfer)
				begin
					if(dxr_in_write)
					begin
						dxr_out <= dxr_in_write_data;
						rb_out_xrdy <= 0;
					end
				end
			end
			default: begin
			end
			endcase	
		end
	
    end

endmodule



