module register_bank(

  input logic reset, clk,   
  input logic [31:0] rb_in_address,
  input logic [31:0] rb_in_write_data,
  input logic rb_in_read,
  input logic rb_in_write,
  input logic rb_in_begin_transfer,
  input logic [3:0] rb_in_byte_enable,
  output logic rb_in_wait_request,
  
  input logic [31:0] general_in_pcr,
    
  input logic receive_in_rrdy,
  input logic receive_in_rfull,
  input logic receive_in_rsyncerr,
  input logic [2:0] receive_in_rcblk,
  
  input logic transmit_in_xsyncerr,
  input logic transmit_in_xempty,
  input logic transmit_in_xrdy,
  input logic [2:0] transmit_in_xcblk,
      
  output logic  [31:0] rb_out_mcr,
  output logic  [31:0] rb_out_rcer0,
  output logic  [31:0] rb_out_rcer1,  
  output logic  [31:0] rb_out_rcer2,  
  output logic  [31:0] rb_out_rcer3,  
  output logic  [31:0] rb_out_xcer0, 
  output logic  [31:0] rb_out_xcer1,
  output logic  [31:0] rb_out_xcer2,
  output logic  [31:0] rb_out_xcer3,
  output logic  [31:0] rb_out_srgr,
  output logic  [31:0] rb_out_xcr,
  output logic  [31:0] rb_out_rcr,
  output logic  [31:0] rb_out_pcr,
  output logic  [31:0] rb_out_spcr,
  
  output logic  [31:0] rb_read_data,
  
  output logic  [31:0] dynamic_reg_rmult_rcer0,
  output logic  [31:0] dynamic_reg_rmult_rcer1,
  output logic  [31:0] dynamic_reg_rmult_rcer2,
  output logic  [31:0] dynamic_reg_rmult_rcer3,
  output logic  [31:0] dynamic_reg_rmult_mcr,
  output logic  dynamic_reg_rmult_request,
  
  output logic  [31:0] dynamic_reg_gp_pcr,
  
  output logic  [31:0] dynamic_reg_tmult_xcer0,  
  output logic  [31:0] dynamic_reg_tmult_xcer1,  
  output logic  [31:0] dynamic_reg_tmult_xcer2,  
  output logic  [31:0] dynamic_reg_tmult_xcer3,  
  output logic  [31:0] dynamic_reg_tmult_rcer0,  
  output logic  [31:0] dynamic_reg_tmult_rcer1,  
  output logic  [31:0] dynamic_reg_tmult_rcer2,  
  output logic  [31:0] dynamic_reg_tmult_rcer3,  
  output logic  [31:0] dynamic_reg_tmult_mcr,
  output logic  dynamic_reg_tmult_request,
  
  output logic ac_srg_active,
  output logic ac_general_rec_active,
  output logic ac_companding_active,
  output logic ac_receive_mult_active,
  output logic ac_general_trans_active,
  output logic ac_transmit_active,
  output logic ac_receive_active,
  output logic ac_transmit_mult_active
  );
  
  logic load_spcr;
  logic load_rcr;
  logic load_xcr;
  logic load_srgr;
  logic load_pcr;
  logic load_mcr;
  logic load_xcere0;
  logic load_xcere1;
  logic load_xcere2;
  logic load_xcere3;
  logic load_rcere0;
  logic load_rcere1;
  logic load_rcere2;
  logic load_rcere3;
  logic load_rrdy;
  logic load_rfull;
  logic load_rsyncerr;
  logic load_rcblk;
  logic load_xsyncerr;
  logic load_xempty;
  logic load_xrdy;
  logic load_xcblk;
  logic load_general_pcr;
  
  logic read_spcr;
  logic read_rcr;
  logic read_xcr;
  logic read_mcr;
  logic read_pcr;
  logic read_srgr;
  logic read_xcere0;
  logic read_xcere1;
  logic read_xcere2;
  logic read_xcere3;
  logic read_rcere0;
  logic read_rcere1;
  logic read_rcere2;
  logic read_rcere3;
  
  logic [31:0] rcere0;
  logic [31:0] rcere0_out;
  logic [31:0] rcere1;
  logic [31:0] rcere1_out;
  logic [31:0] rcere2;
  logic [31:0] rcere2_out;
  logic [31:0] rcere3;
  logic [31:0] rcere3_out;
  
  logic [31:0] xcere0;
  logic [31:0] xcere0_out;
  logic [31:0] xcere1;
  logic [31:0] xcere1_out;
  logic [31:0] xcere2;
  logic [31:0] xcere2_out;
  logic [31:0] xcere3;
  logic [31:0] xcere3_out;
  
  logic [31:0] spcr;
  logic [31:0] spcr_out;
  logic [31:0] pcr;
  logic [31:0] pcr_out;
  logic [31:0] mcr;
  logic [31:0] mcr_out;
  logic [31:0] srgr;
  logic [31:0] srgr_out;
  logic [31:0] xcr;
  logic [31:0] xcr_out;
  logic [31:0] rcr;
  logic [31:0] rcr_out;
  
  logic read_d_pcr;
  logic read_d_mcr_r;
  logic read_d_mcr_t;
  logic read_d_xcere0;
  logic read_d_xcere1;
  logic read_d_xcere2;
  logic read_d_xcere3;
  logic read_d_rcere0;
  logic read_d_rcere1;
  logic read_d_rcere2;
  logic read_d_rcere3;
  
  SPCR spcr_i(.reset(reset), .clk(clk), .load_spcr(load_spcr),
	.spcr(rb_in_write_data), .receive_in_rrdy(receive_in_rrdy),
	.receive_in_rfull(receive_in_rfull), .receive_in_rsyncerr(receive_in_rsyncerr),
	.transmit_in_xsyncerr(transmit_in_xsyncerr), .transmit_in_xempty(transmit_in_xempty),
	.transmit_in_xrdy(transmit_in_xrdy), .load_rrdy(load_rrdy), .load_rfull(load_rfull),
	.load_rsyncerr(load_rsyncerr), .load_xsyncerr(load_xsyncerr), .load_xempty(load_xempty),
	.load_xrdy(load_xempty), .rb_out_spcr(rb_out_spcr));
	
  RCR rcr_i(.reset(reset), .clk(clk), .load_rcr(load_rcr),
	.rcr(rb_in_write_data), .rb_out_rcr(rb_out_rcr));
	
  XCR xcr_i(.reset(reset), .clk(clk), .load_xcr(load_xcr),
	.xcr(rb_in_write_data), .rb_out_xcr(rb_out_xcr));
	
  SRGR srgr_i(.reset(reset), .clk(clk), .load_srgr(load_srgr),
	.srgr(rb_in_write_data), .rb_out_srgr(rb_out_srgr));
	
  PCR pcr_i(.reset(reset), .clk(clk), .load_pcr(load_pcr),
	.pcr(rb_in_write_data), .general_in_pcr(general_in_pcr),
	.load_general_pcr(load_general_pcr), .rb_out_pcr(rb_out_pcr),
	.dynamic_reg_gp_pcr(dynamic_reg_gp_pcr), .read_d_pcr(read_d_pcr));
	
  MCR mcr_i(.reset(reset), .clk(clk), .load_mcr(load_mcr),
	.mcr(rb_in_write_data), .receive_in_rcblk(receive_in_rcblk),
	.transmit_in_xcblk(transmit_in_xcblk), .load_rcblk(load_rcblk),
	.dynamic_reg_rmult_mcr(dynamic_reg_rmult_mcr), .dynamic_reg_tmult_mcr(dynamic_reg_tmult_mcr), 
	.load_xcblk(load_xcblk), .rb_out_mcr(rb_out_mcr), .read_d_mcr_r(read_d_mcr_r),
	.read_d_mcr_t(read_d_mcr_t));
	
  XCERE0 xcere0_i(.reset(reset), .clk(clk), .load_xcere0(load_xcere0),
	.xcere0(rb_in_write_data), .rb_out_xcer0(rb_out_xcer0), 
	.dynamic_reg_tmult_xcer0(dynamic_reg_tmult_xcer0),    
	.read_d_xcere0(read_d_xcere0));
	
  XCERE1 xcere1_i(.reset(reset), .clk(clk), .load_xcere1(load_xcere1),
	.xcere1(rb_in_write_data), .rb_out_xcer1(rb_out_xcer1),
    .dynamic_reg_tmult_xcer1(dynamic_reg_tmult_xcer1),
    .read_d_xcere1(read_d_xcere1));
	
  XCERE2 xcere2_i(.reset(reset), .clk(clk), .load_xcere2(load_xcere2),
	.xcere2(rb_in_write_data), .rb_out_xcer2(rb_out_xcer2),
    .read_d_xcere2(read_d_xcere2), .dynamic_reg_tmult_xcer2(dynamic_reg_tmult_xcer2));
	
  XCERE3 xcere3_i(.reset(reset), .clk(clk), .load_xcere3(load_xcere3),
	.xcere3(rb_in_write_data), .rb_out_xcer3(rb_out_xcer3),
    .read_d_xcere3(read_d_xcere3), .dynamic_reg_tmult_xcer3(dynamic_reg_tmult_xcer3));
	
  RCERE0 rcere0_i(.reset(reset), .clk(clk), .load_rcere0(load_rcere0),
	.rcere0(rb_in_write_data), .rb_out_rcer0(rb_out_rcer0),
	.dynamic_reg_rmult_rcer0(dynamic_reg_rmult_rcer0),
	.dynamic_reg_tmult_rcer0(dynamic_reg_tmult_rcer0),
	.read_d_rcere0(read_d_rcere0));
	
  RCERE1 rcere1_i(.reset(reset), .clk(clk), .load_rcere1(load_rcere1),
	.rcere1(rb_in_write_data), .rb_out_rcer1(rb_out_rcer1),
    .dynamic_reg_rmult_rcer1(dynamic_reg_rmult_rcer1),
	.dynamic_reg_tmult_rcer1(dynamic_reg_tmult_rcer1),
    .read_d_rcere1(read_d_rcere1));
	
  RCERE2 rcere2_i(.reset(reset), .clk(clk), .load_rcere2(load_rcere2),
	.rcere2(rb_in_write_data), .rb_out_rcer2(rb_out_rcer2),
    .dynamic_reg_rmult_rcer2(dynamic_reg_rmult_rcer2),
	.dynamic_reg_tmult_rcer2(dynamic_reg_tmult_rcer2),
    .read_d_rcere2(read_d_rcere2));
	
  RCERE3 rcere3_i(.reset(reset), .clk(clk), .load_rcere3(load_rcere3),
	.rcere3(rb_in_write_data), .rb_out_rcer3(rb_out_rcer3),
    .dynamic_reg_rmult_rcer3(dynamic_reg_rmult_rcer3),
	.dynamic_reg_tmult_rcer3(dynamic_reg_tmult_rcer3),
    .read_d_rcere3(read_d_rcere3)); 
	
  MUX_read_data MUX_read_data_i(.reset(reset), .read_spcr(read_spcr),
	.read_rcr(read_rcr), .read_xcr(read_xcr), .read_mcr(read_mcr),
	.read_pcr(read_pcr), .read_srgr(read_srgr), .read_xcere0(read_xcere0),
	.read_xcere1(read_xcere1), .read_xcere2(read_xcere2), .read_xcere3(read_xcere3),
	.read_rcere0(read_rcere0), .read_rcere1(read_rcere1), .read_rcere2(read_rcere2),
	.read_rcere3(read_rcere3), .rb_out_spcr(rb_out_spcr), .rb_out_rcr(rb_out_rcr),
	.rb_out_xcr(rb_out_xcr), .rb_out_mcr(rb_out_mcr), .rb_out_pcr(rb_out_pcr),
	.rb_out_srgr(rb_out_srgr), .rb_out_xcer0(rb_out_xcer0), .rb_out_xcer1(rb_out_xcer1),
	.rb_out_xcer2(rb_out_xcer2), .rb_out_xcer3(rb_out_xcer3), .rb_out_rcer0(rb_out_rcer0),
	.rb_out_rcer1(rb_out_rcer1), .rb_out_rcer2(rb_out_rcer2), .rb_out_rcer3(rb_out_rcer3),
	.rb_read_data(rb_read_data));
	
  control_rb control_rb_i(.clk(clk), .reset(reset), .rb_in_address(rb_in_address),
        .rb_in_write_data(rb_in_write_data), .rb_in_read(rb_in_read),
        .rb_in_write(rb_in_write), .rb_in_begin_transfer(rb_in_begin_transfer),
        .rb_in_byte_enable(rb_in_byte_enable), .rb_in_wait_request(rb_in_wait_request), 
	    .ac_srg_active(ac_srg_active), 
		.ac_companding_active(ac_companding_active), .ac_receive_mult_active(ac_receive_mult_active),
        .ac_general_rec_active(ac_general_rec_active), .ac_general_trans_active(ac_general_trans_active),
        .ac_transmit_active(ac_transmit_active), .ac_receive_active(ac_receive_active),
        .ac_transmit_mult_active(ac_transmit_mult_active), .load_spcr(load_spcr),
        .load_rcr(load_rcr), .load_xcr(load_xcr), .load_mcr(load_mcr),
        .load_pcr(load_pcr), .load_srgr(load_srgr), .load_xcere0(load_xcere0),
        .load_xcere1(load_xcere1), .load_xcere2(load_xcere2), .load_xcere3(load_xcere3),
        .load_rcere0(load_rcere0), .load_rcere1(load_rcere1), .load_rcere2(load_rcere2),
        .load_rcere3(load_rcere3), .load_rrdy(load_rrdy), .load_rfull(load_rfull),
        .load_rsyncerr(load_rsyncerr), .load_rcblk(load_rcblk), .load_xsyncerr(load_xsyncerr),
        .load_xempty(load_xempty), .load_xrdy(load_xrdy), .load_xcblk(load_xcblk),
        .read_spcr(read_spcr), .read_rcr(read_rcr), .read_xcr(read_xcr),
        .read_mcr(read_mcr), .read_pcr(read_pcr), .read_srgr(read_srgr),
        .read_xcere0(read_xcere0), .read_xcere1(read_xcere1), .read_xcere2(read_xcere2),
        .read_xcere3(read_xcere3), .read_rcere0(read_rcere0), .read_rcere1(read_rcere1),
        .read_rcere2(read_rcere2), .read_rcere3(read_rcere3), .load_general_pcr(load_general_pcr),        
        .rmult_request(dynamic_reg_rmult_request), .tmult_request(dynamic_reg_tmult_request),
        .read_d_rcere0(read_d_rcere0), .read_d_xcere0(read_d_xcere0),
        .read_d_rcere1(read_d_rcere1), .read_d_xcere1(read_d_xcere1),
        .read_d_rcere2(read_d_rcere2), .read_d_xcere2(read_d_xcere2),
        .read_d_rcere3(read_d_rcere3), .read_d_xcere3(read_d_xcere3),
        .read_d_mcr_t(read_d_mcr_t), .read_d_mcr_r(read_d_mcr_r),
        .read_d_pcr(read_d_pcr));
  

endmodule