module srg (
input ac_srg_active,
input clkx,
input clkr,
input fsx,
input fsr,
input clks,
input cpu_clk,

input [31:0] rb_in_srgr,
input [31:0] rb_in_spcr,
input [31:0] rb_in_pcr,

input copy_in_copy,


output logic srg_rec_out_clkr_int,
output logic srg_trans_out_clkx_int,
output logic pins_out_clkr,
output logic pins_out_clkx,

output logic srg_rec_out_fsr_int,
output logic srg_trans_out_fsx_int,
output logic pins_out_fsr,
output logic pins_out_fsx
);


logic [31:0] rb_in_srgr_var;
logic [31:0] rb_in_spcr_var;
logic [31:0] rb_in_pcr_var;

 // PCR
logic clkrp;
logic clkrm;
logic clkxp;
logic clkxm;

logic fsrp;
logic fsrm;
logic fsxp;
logic fsxm;

logic rioen;
logic xioen;

// SRGR
logic gsync;

logic clksp;
logic [7:0] clkgdv;
logic clksm;

logic fsgm;
logic [11:0] fper;
logic [7:0] fwid;

// SPCR
logic frst;
logic grst;
logic [1:0] clkstp;
logic dlb;

// Srg_core fios
logic clkg;
logic fsg;
		
		//Fios muxes
logic mux_fsrm_out;
logic mux_fsxm_out;
logic mux_fsgm_out;
logic mux_dlb_frame_out;
logic mux_dlb_clk_out;
logic mux_clkrm_out;
logic mux_clkxm_out;


		//Fios xor's
logic xor_fsr_out;
logic xor_fsx_out;
logic xor_fsxp_out;
logic xor_fsrp_out;
logic xor_clkx_out;
logic xor_clkr_out;
logic xor_clkxp_out;
logic xor_clkrp_out;

//always #10
//begin
//	$display("<----------- %d      ------------------------------->", clkg);
//end

always_ff @(posedge ac_srg_active)
begin
	rb_in_srgr_var[31:0] = rb_in_srgr[31:0];
	rb_in_spcr_var[31:0] = rb_in_spcr[31:0];
	rb_in_pcr_var[31:0] = rb_in_pcr[31:0];	
end

always_comb
begin

//PCR
	clkrp = rb_in_pcr[0];
	clkrm = rb_in_pcr[8];
	clkxp = rb_in_pcr[1];
	clkxm = rb_in_pcr[9];
	fsrp = rb_in_pcr[2];
	fsrm = rb_in_pcr[10];
	fsxp = rb_in_pcr[3];
	fsxm = rb_in_pcr[11];
	rioen = /*rb_in_pcr[12]_var*/ 1'b0;
	xioen = /*rb_in_pcr[13]_var*/ 1'b0;

// SRGR
	gsync = rb_in_srgr[31];
	clksp = rb_in_srgr[30];
	clkgdv = rb_in_srgr[7:0];
	clksm = rb_in_srgr[29];
	fsgm = rb_in_srgr[28];
	fper = rb_in_srgr[27:16];
	fwid = rb_in_srgr[15:8];

// SPCR
	frst = rb_in_spcr[23];
	grst = rb_in_spcr[22];
	clkstp = rb_in_spcr[12:11];
	dlb = rb_in_spcr[15];

// SRG em si

// --> Xor's entrada frame

	xor_fsr_out = (fsrp ^ fsr);
	xor_fsx_out = (fsxp ^ fsx);

	
	
// ---> Muxes entrada frame

	if (fsgm) // Mux FSGM
	begin
		mux_fsgm_out = fsg;
	end
	else
	begin
		mux_fsgm_out = copy_in_copy;
	end

	if (fsxm) // Mux FSXM
	begin
		mux_fsxm_out = mux_fsgm_out;
	end
	else
	begin
		mux_fsxm_out = xor_fsx_out;
	end

	if (fsrm)  // Mux FSRM
	begin
		mux_fsrm_out = fsg;
	end
	else
	begin
		mux_fsrm_out = xor_fsr_out;
	end
	
	if (dlb) // Mux DLB do frame
	begin
		mux_dlb_frame_out = mux_fsxm_out;
	end
	else
	begin
		mux_dlb_frame_out = mux_fsrm_out;
	end
	

	
// --> Xor's saida frame
	xor_fsrp_out = (fsrp ^ (mux_dlb_frame_out & (!rioen)));  // obs.: ver inset
	xor_fsxp_out = (fsxp ^ (mux_fsxm_out & (!xioen)));  // obs.: ver inset
	
// Muxes saida frame

	if (fsxm && ac_srg_active) // "Blocador"
	begin
		pins_out_fsx = xor_fsxp_out;
	end
	else
	begin
		pins_out_fsx = 1'b0;
	end
	
	if (fsrm && !gsync && ac_srg_active) // "Blocador"
	begin
		pins_out_fsr = xor_fsrp_out;
	end
	else
	begin
		pins_out_fsr = 1'b0;
	end

// Xor's entrada clock

	xor_clkx_out = (clkxp ^ clkx);
	xor_clkr_out = (clkr ^ clkrp);

// Muxes entrada clock



	if (clkxm) // Mux clkxm
	begin
		mux_clkxm_out = clkg;
	end
	else
	begin
		mux_clkxm_out = xor_clkx_out;
	end
	
	if (clkrm) // Mux clkrm
	begin
		mux_clkrm_out = clkg;
	end
	else
	begin
		mux_clkrm_out = xor_clkr_out;
	end
	
	
	if (dlb) // Mux DLB do clock de entrada
	begin
		mux_dlb_clk_out = mux_clkxm_out;
	end
	else
	begin
		mux_dlb_clk_out = mux_clkrm_out;
	end
	


	
// Xor's de saida clock

	xor_clkxp_out = (clkxp ^ (mux_clkxm_out & (!xioen)));  // obs.: ver inset
	xor_clkrp_out = (clkrp ^ (mux_dlb_clk_out & (!rioen)));  // obs.: ver inset
	
// Muxes saida clock

	if (clkxm && ac_srg_active) // "Blocador"
	begin
		pins_out_clkx = xor_clkxp_out;
	end
	else
	begin
		pins_out_clkx = 1'b0;
	end
	
	if (clkrm && ac_srg_active) // "Blocador"
	begin
		pins_out_clkr = xor_clkrp_out;
	end
	else
	begin
		pins_out_clkr = 1'b0;
	end
	
// Ligacao dos fios internos
	if(ac_srg_active)
	begin
		srg_rec_out_fsr_int = mux_dlb_frame_out;
		srg_trans_out_fsx_int = mux_fsxm_out;
		srg_rec_out_clkr_int = mux_dlb_clk_out;
		srg_trans_out_clkx_int = mux_clkxm_out;
	end
	else
	begin
		srg_rec_out_fsr_int = 1'b0;
		srg_trans_out_fsx_int = 1'b0;
		srg_rec_out_clkr_int = 1'b0;
		srg_trans_out_clkx_int = 1'b0;
	end
end

srg_core srg_core_i (
.activate(ac_srg_active),
.frst(frst),
.grst(grst),
.clks(clks),
.clk_int(cpu_clk),
.fsr_int(xor_fsrp_out),
.clksp(clksp),
.gsync(gsync),
.clksm(clksm),
.clkgdv(clkgdv),
.fper(fper),
.fwid(fwid),
.fsg(fsg),
.clkg (clkg)

);

endmodule
