
module srg_core(
        input activate,
        input frst,
        input grst,
        input clks,
        input clk_int,
        input fsr_int,
        input clksp,
        input gsync,
        input clksm,
        input [7:0] clkgdv,
        input [11:0] fper,
        input [7:0] fwid,
        output logic fsg,
        output logic clkg
    );

    logic signed [7:0] div_high;
    logic signed [7:0] div_low;
    logic signed [7:0] count_high;
    logic signed [7:0] count_low;
    logic signed [15:0] div_fwid;
    logic signed [19:0] div_fper;
    logic signed [19:0] count_fper;
    logic signed [15:0] count_fwid;

    logic clkg_reg;
    logic resync;
    logic fsr_reg;
    logic clksrg;
    logic gsync_fio;
    logic activate_reg;

    always_comb
    begin
        div_low = (clkgdv / 2) - ((clkgdv + 1) % 2);
        div_high = (clkgdv / 2);
        div_fwid = ((fwid + 1) * (clkgdv + 1)) - 1;
        div_fper = ((fper + 1) * (clkgdv + 1)) - 1;
	
	
        if(clksm == 1)
        begin
            clksrg = clk_int;
            gsync_fio = 0;
		
        end
        else
        begin
            clksrg = (clks ^ clksp);
            gsync_fio = gsync;
        end
	
        if (clkgdv != 0)
        begin
            clkg = clkg_reg;
        end
        else
        begin
            clkg = clksrg;
        end
	
	

    end

    always_ff @(posedge clksrg)
    begin
        fsr_reg <= fsr_int;
        activate_reg <= activate;
        if(gsync_fio == 1 && fsr_reg == 0 && fsr_int == 1)
        begin
            clkg_reg <= 1;
            count_high <= 0;
            count_low <= 0;
            fsg <= 1;
            count_fper <= 0;
            count_fwid <= 0;
			
            resync <= 1;
        end
        else
        begin
            resync <= 0;
        
            if(activate_reg)
            begin
            	
            	
            	
                if(grst)
                begin
                    case(clkg_reg)
                        1:
                        if(count_high < div_high && div_high > 0)
                        begin
                            count_high <= count_high + 1;
                        end
                        else
                        begin
                            count_high <= 0;
                            clkg_reg <= 0;
                        end
                        0:
                        if(count_low < div_low && div_low > 0)
                        begin
                            count_low <= count_low + 1;
                        end
                        else
                        begin
                            count_low <= 0;
                            clkg_reg <= 1;
                        end
                        default:
                        clkg_reg <= 0;
                    endcase
                end
                else
                begin
                    clkg_reg <= 0;
                    count_high <= 0;
                    count_low <= div_low;
                end
                
                if(frst && grst)
                begin
                    if(count_fper < div_fper)
                    begin
                        count_fper <= count_fper + 1;
                    end
                    else if (gsync_fio == 0)
                    begin
                        count_fper <= 0;
                        fsg <= 1;
                    end
                    else
                    begin
                        count_fper <= 0;
                    end
			
			
                    if (fsg && count_fwid < div_fwid )
                    begin
                        count_fwid <= count_fwid + 1;
                    end
                    else if ( (count_fper < div_fper || fsg == 1 ) && !resync )
                    begin
                        count_fwid <= 0;
                        fsg <= 0;
                    end
                end
                else
                begin
                    fsg <= 0;
                    count_fwid <= 0;
                    count_fper <= div_fper - (clkgdv + 1);
                end
            end
            else
            begin
                fsg <= 0;
                clkg_reg <= 0;
                count_fwid <= 0;
                count_fper <= div_fper - (clkgdv + 1);
                count_high <= 0;
                count_low <= div_low;
            end
	
        end
	
    end


endmodule
