module transmit(
 input reset, clk, 
  input clkx, fsx,  
  input ac_transmit_active,

 
  input  [31:0] rb_in_spcr,
  
  
  input  [31:0] rb_in_pcr,
  input  [31:0] rb_in_rcr,
  input  [31:0] rb_in_xcr,
  input  [31:0] rb_in_srgr,
  input  [31:0] rb_in_mcr,
  input  [31:0] rb_in_rcer0,
  input  [31:0] rb_in_rcer1,
  input  [31:0] rb_in_rcer2,   
  input  [31:0] rb_in_rcer3,
  input  [31:0] rb_in_xcer0,
  input  [31:0] rb_in_xcer1,
  input  [31:0] rb_in_xcer2,
  input  [31:0] rb_in_xcer3,
   

  input  [31:0] dxr_in_address,
  input  [31:0] dxr_in_write_data,

  input dxr_in_write,
  input dxr_in_begin_transfer,
  input [3:0] dxr_in_byte_enable, 

  output logic dxr_in_wait_request,
  
  input  [31:0] mult_in_unmasked_channel1,  
  input  [31:0] mult_in_unmasked_channel2,
  input  [31:0] mult_in_unmasked_channel3,
  input  [31:0] mult_in_unmasked_channel4,
  input  [31:0] mult_in_enabled_channel1,
  input  [31:0] mult_in_enabled_channel2,
  input  [31:0] mult_in_enabled_channel3,
  input  [31:0] mult_in_enabled_channel4,
  

  
  output logic rb_out_xsyncerr,
  output logic rb_out_xempty,
  output logic rb_out_xrdy,
  output logic  [2:0] rb_out_xcblk,
 
  output logic dx_out_dx,
  output logic dxr_int_out_xint,
  output logic dxr_int_out_xevt,
 
  output logic  [31:0] dlb_out_data,



  output logic copy_out_copy

  


);
    

  logic [2:0] state; 
  logic [2:0] next;
  logic [4:0] bit_count;
  logic [6:0] element_count;

	
	logic [4:0] xwdlen1_decod;
	
	logic saida;
	
	assign dx_out_dx = saida;
	assign dlb_out_data = saida;
	
	logic [31:0] xsr;
	
	logic [31:0] dxr_out;
	
    parameter 	DISABLED = 0,
				ENABLED = 1,
				INITIAL_TRANSFER = 2,
				SERIAL_TRANSFER = 3,
				WAIT1 = 4;
//				ABORT = 6;


	//SPCR
	logic [1:0] xintm;
	logic xsyncerr;
	logic xempty;
	logic xrdy;
	logic xrst;
	logic dlb;
	
	//XCR
	logic xphase;
	logic [6:0] xfrlen2;
	logic [2:0] xwdlen2;
	logic [1:0] xcompand;
	logic xfig;
	logic [1:0] xdatdly;
	logic [6:0] xfrlen1;
	logic [2:0] xwdlen1;
	logic xwdrevrs;
	logic [2:0] xwdlen1_var;

	always_comb
	begin
		//SPCR
		xintm = rb_in_spcr[21:20];
		xsyncerr = rb_in_spcr[19];
		xempty = rb_in_spcr[18];
		xrdy = rb_in_spcr[17];
		xrst = rb_in_spcr[16];
		dlb = rb_in_spcr[15];

		//XCR
		xphase = rb_in_xcr[31];
		xfrlen2 = rb_in_xcr[30:24];
		xwdlen2 = rb_in_xcr[23:21];
		xcompand = rb_in_xcr[20:19];
		xfig = rb_in_xcr[18];
		xdatdly = rb_in_xcr[17:16];
		xfrlen1 = rb_in_xcr[14:8];
		xwdlen1 = rb_in_xcr[7:5];
		xwdrevrs = rb_in_xcr[4];
		
		

	end
	
	dxr dxr_i(.clkx(clkx), .reset(reset), .copy_out_copy(copy_out_copy),
			  .dxr_in_write_data(dxr_in_write_data), .dxr_in_write(dxr_in_write),
			  .dxr_in_begin_transfer(dxr_in_begin_transfer), .dxr_in_byte_enable(dxr_in_byte_enable), 
			  .dxr_in_wait_request(dxr_in_wait_request), .rb_out_xrdy(rb_out_xrdy),
			  .dxr_out(dxr_out));
	
	always_comb
	begin
		case(xwdlen1_var)
			0:
				xwdlen1_decod = 8;
			1:
				xwdlen1_decod = 12;
			2:
				xwdlen1_decod = 16;
			3:
				xwdlen1_decod = 20;
			4:
				xwdlen1_decod = 24;
			5:
				xwdlen1_decod = 32;
			default:
				xwdlen1_decod = 8;
		endcase
	end
	
	
	always_comb
	begin
		
		case(state)
			DISABLED:
			begin
				if(ac_transmit_active == 0)
				begin
					next = DISABLED;
				end
				else
				begin
					next = ENABLED;
				end
			end
			ENABLED:
			begin
				if(fsx == 0)
				begin
					next = ENABLED;
				end
				else
				begin
					if(xdatdly == 0)
					begin
						next = SERIAL_TRANSFER;
					end
					else if(xdatdly == 1)
					begin
						next = INITIAL_TRANSFER;
					end
					else if(xdatdly == 2)
					begin
						next = WAIT1;
					end
					else
					begin
						next = ENABLED;
					end
				end
			end
			WAIT1:
			begin
				next = INITIAL_TRANSFER;
			end
			INITIAL_TRANSFER:
			begin
				next = SERIAL_TRANSFER;
			end
			SERIAL_TRANSFER:
			begin
				if(element_count == 0)
				begin
					next = ENABLED;
				end
				else if(bit_count == 0)
				begin
					next = INITIAL_TRANSFER;
				end
				else
				begin
					next = SERIAL_TRANSFER;
				end
			end
			default:
			begin
				next = DISABLED;
			end
		
		endcase
	
	end
	
	
	always_ff @(posedge clkx)
	begin
		if(!reset) begin
			copy_out_copy <= 0;
			dxr_int_out_xint <= 0;
			saida <= 0;
			bit_count <= 0;
			element_count <= 0;
			xwdlen1_var <= 0;
			xsr <= 0;
		end
		else begin
		state <= next;
		copy_out_copy <= 0;
		dxr_int_out_xint <= 0;
		
			case(state)
			DISABLED:
			begin
				saida <= 0;
				bit_count <= xwdlen1_decod - 1;
				element_count <= xfrlen1+1;
				xwdlen1_var <= xwdlen1;
			end
			ENABLED:
			begin
				element_count <= xfrlen1+1;
				xwdlen1_var <= xwdlen1;
				saida <= 0;
			
				if(next == SERIAL_TRANSFER)
				begin
					saida <= dxr_out[bit_count];
					xsr <= dxr_out;
					bit_count <= xwdlen1_decod - 2;
					
					if (xintm == 0)
					begin
						copy_out_copy <= 1;
						dxr_int_out_xint <= 1;
					end
			
				end
			end
			WAIT1:
			begin
				//nada
			end
			INITIAL_TRANSFER:
			begin
				saida <= dxr_out[bit_count];
				xsr <= dxr_out;
				copy_out_copy <= 1;
				dxr_int_out_xint <= 1;
				bit_count <= xwdlen1_decod - 2;
				//element_count <= element_count - 1;
			end
			SERIAL_TRANSFER:
			begin
				saida <= xsr[bit_count];
				if(bit_count != 0)
				begin
					bit_count <= bit_count - 1;
				end
				else
				begin
					bit_count <= xwdlen1_decod - 1;
				end
				if (bit_count == 1)
				begin
					element_count <= element_count - 1;
				end
			end
			default:
			begin
			end
			
		endcase
		end
	end


endmodule
